@@ -4,11 +4,9 @@
 # can be used by non-unix systems that do not have built-in support
 # for shells.
 #
-# RCS: @(#) $Id: console.tcl,v 1.22.2.7 2007/11/09 07:08:51 das Exp $
-#
 # Copyright (c) 1995-1997 Sun Microsystems, Inc.
 # Copyright (c) 1998-2000 Ajuba Solutions.
-# Copyright (c) 2007 Daniel A. Steffen <das@users.sourceforge.net>
+# Copyright (c) 2007-2008 Daniel A. Steffen <das@users.sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -22,10 +20,9 @@ namespace eval ::tk::console {
     variable magicKeys   1   ; # enable brace matching and proc/var recognition
     variable maxLines    600 ; # maximum # of lines buffered in console
     variable showMatches 1   ; # show multiple expand matches
-
+    variable useFontchooser [llength [info command ::tk::fontchooser]]
     variable inPlugin [info exists embed_args]
-    variable defaultPrompt  ; # default prompt if tcl_prompt1 isn't used
-
+    variable defaultPrompt   ; # default prompt if tcl_prompt1 isn't used
 
     if {$inPlugin} {
 	set defaultPrompt {subst {[history nextid] % }}
@@ -44,80 +41,121 @@ interp alias {} EvalAttached {} consoleinterp eval
 # 	None.
 
 proc ::tk::ConsoleInit {} {
-    global tcl_platform
-
     if {![consoleinterp eval {set tcl_interactive}]} {
 	wm withdraw .
     }
 
-    if {$tcl_platform(platform) eq "macintosh"
-	    || [tk windowingsystem] eq "aqua"} {
+    if {[tk windowingsystem] eq "aqua"} {
 	set mod "Cmd"
     } else {
 	set mod "Ctrl"
     }
 
-    if {[catch {menu .menubar} err]} { bgerror "INIT: $err" }
-    .menubar add cascade -label File -menu .menubar.file -underline 0
-    .menubar add cascade -label Edit -menu .menubar.edit -underline 0
+    if {[catch {menu .menubar} err]} {
+	bgerror "INIT: $err"
+    }
+    AmpMenuArgs .menubar add cascade -label [mc &File] -menu .menubar.file
+    AmpMenuArgs .menubar add cascade -label [mc &Edit] -menu .menubar.edit
 
     menu .menubar.file -tearoff 0
-    .menubar.file add command -label [mc "Source..."] \
-	    -underline 0 -command tk::ConsoleSource
-    .menubar.file add command -label [mc "Hide Console"] \
-	    -underline 0 -command {wm withdraw .}
-    .menubar.file add command -label [mc "Clear Console"] \
-	    -underline 0 -command {.console delete 1.0 "promptEnd linestart"}
-   if {$tcl_platform(platform) eq "macintosh"
-	    || [tk windowingsystem] eq "aqua"} {
-	.menubar.file add command -label [mc "Quit"] \
-		-command exit -accel Cmd-Q
-    } else {
-	.menubar.file add command -label [mc "Exit"] \
-		-underline 1 -command exit
+    AmpMenuArgs .menubar.file add command -label [mc "&Source..."] \
+	    -command {tk::ConsoleSource}
+    AmpMenuArgs .menubar.file add command -label [mc "&Hide Console"] \
+	    -command {wm withdraw .}
+    AmpMenuArgs .menubar.file add command -label [mc "&Clear Console"] \
+	    -command {.console delete 1.0 "promptEnd linestart"}
+    if {[tk windowingsystem] ne "aqua"} {
+	AmpMenuArgs .menubar.file add command -label [mc E&xit] -command {exit}
     }
 
     menu .menubar.edit -tearoff 0
-    .menubar.edit add command -label [mc "Cut"] -underline 2 \
-	    -command { event generate .console <<Cut>> } -accel "$mod+X"
-    .menubar.edit add command -label [mc "Copy"] -underline 0 \
-	    -command { event generate .console <<Copy>> } -accel "$mod+C"
-    .menubar.edit add command -label [mc "Paste"] -underline 1 \
-	    -command { event generate .console <<Paste>> } -accel "$mod+V"
+    AmpMenuArgs	.menubar.edit add command -label [mc Cu&t]   -accel "$mod+X"\
+	    -command {event generate .console <<Cut>>}
+    AmpMenuArgs	.menubar.edit add command -label [mc &Copy]  -accel "$mod+C"\
+	    -command {event generate .console <<Copy>>}
+    AmpMenuArgs	.menubar.edit add command -label [mc P&aste] -accel "$mod+V"\
+	    -command {event generate .console <<Paste>>}
 
-    if {$tcl_platform(platform) ne "windows"} {
-	.menubar.edit add command -label [mc "Clear"] -underline 2 \
-		-command { event generate .console <<Clear>> }
+    if {[tk windowingsystem] ne "win32"} {
+	AmpMenuArgs .menubar.edit add command -label [mc Cl&ear] \
+		-command {event generate .console <<Clear>>}
     } else {
-	.menubar.edit add command -label [mc "Delete"] -underline 0 \
-		-command { event generate .console <<Clear>> } -accel "Del"
-	
-	.menubar add cascade -label Help -menu .menubar.help -underline 0
+	AmpMenuArgs .menubar.edit add command -label [mc &Delete] \
+		-command {event generate .console <<Clear>>} -accel "Del"
+
+	AmpMenuArgs .menubar add cascade -label [mc &Help] -menu .menubar.help
 	menu .menubar.help -tearoff 0
-	.menubar.help add command -label [mc "About..."] \
-		-underline 0 -command tk::ConsoleAbout
+	AmpMenuArgs .menubar.help add command -label [mc &About...] \
+		-command tk::ConsoleAbout
+    }
+
+    AmpMenuArgs .menubar.edit add separator
+    if {$::tk::console::useFontchooser} {
+	if {[tk windowingsystem] eq "aqua"} {
+	    .menubar.edit add command -label tk_choose_font_marker
+	    set index [.menubar.edit index tk_choose_font_marker]
+	    .menubar.edit entryconfigure $index \
+		-label [mc "Show Fonts"]\
+		-accelerator "$mod-T"\
+		-command [list ::tk::console::FontchooserToggle]
+	    bind Console <<TkFontchooserVisibility>> \
+		[list ::tk::console::FontchooserVisibility $index]
+	    ::tk::console::FontchooserVisibility $index
+	} else {
+	    AmpMenuArgs .menubar.edit add command -label [mc "&Font..."] \
+		-command [list ::tk::console::FontchooserToggle]
+	}
+	bind Console <FocusIn>  [list ::tk::console::FontchooserFocus %W 1]
+	bind Console <FocusOut> [list ::tk::console::FontchooserFocus %W 0]
+    }
+    AmpMenuArgs .menubar.edit add command -label [mc "&Increase Font Size"] \
+	-accel "$mod++" -command {event generate .console <<Console_FontSizeIncr>>}
+    AmpMenuArgs .menubar.edit add command -label [mc "&Decrease Font Size"] \
+	-accel "$mod+-" -command {event generate .console <<Console_FontSizeDecr>>}
+    AmpMenuArgs .menubar.edit add command -label [mc "Fit To Screen Width"] \
+	-command {event generate .console <<Console_FitScreenWidth>>}
+
+    if {[tk windowingsystem] eq "aqua"} {
+	.menubar add cascade -label [mc Window] -menu [menu .menubar.window]
+	.menubar add cascade -label [mc Help] -menu [menu .menubar.help]
     }
 
     . configure -menu .menubar
 
-    set con [text .console  -yscrollcommand [list .sb set] -setgrid true]
-    scrollbar .sb -command [list $con yview]
-    pack .sb -side right -fill both
-    pack $con -fill both -expand 1 -side left
-    switch -exact $tcl_platform(platform) {
-	"macintosh" {
-	    $con configure -font {Monaco 10 normal} -highlightthickness 0
-	}
-	"windows" {
-	    $con configure -font systemfixed
-	}
-	"unix" {
-	    if {[tk windowingsystem] eq "aqua"} {
-		$con configure -font {Monaco 10 normal} -highlightthickness 0
-	    }
+    # See if we can find a better font than the TkFixedFont
+    catch {font create TkConsoleFont {*}[font configure TkFixedFont]}
+    set families [font families]
+    switch -exact -- [tk windowingsystem] {
+	aqua { set preferred {Monaco 10} }
+	win32 { set preferred {ProFontWindows 8 Consolas 8} }
+	default { set preferred {} }
+    }
+    foreach {family size} $preferred {
+	if {$family in $families} {
+	    font configure TkConsoleFont -family $family -size $size
+	    break
 	}
     }
 
+    # Provide the right border for the text widget (platform dependent).
+    ::ttk::style layout ConsoleFrame {
+	Entry.field -sticky news -border 1 -children {
+	    ConsoleFrame.padding -sticky news
+	}
+    }
+    ::ttk::frame .consoleframe -style ConsoleFrame
+
+    set con [text .console -yscrollcommand [list .sb set] -setgrid true \
+		 -borderwidth 0 -highlightthickness 0 -font TkConsoleFont]
+    if {[tk windowingsystem] eq "aqua"} {
+	scrollbar .sb -command [list $con yview]
+    } else {
+	::ttk::scrollbar .sb -command [list $con yview]
+    }
+    pack .sb  -in .consoleframe -fill both -side right -padx 1 -pady 1
+    pack $con -in .consoleframe -fill both -expand 1 -side left -padx 1 -pady 1
+    pack .consoleframe -fill both -expand 1 -side left
+
     ConsoleBind $con
 
     $con tag configure stderr	-foreground red
@@ -131,6 +169,9 @@ proc ::tk::ConsoleInit {} {
 
     focus $con
 
+    # Avoid listing this console in [winfo interps]
+    if {[info command ::send] eq "::send"} {rename ::send {}}
+
     wm protocol . WM_DELETE_WINDOW { wm withdraw . }
     wm title . [mc "Console"]
     flush stdout
@@ -152,7 +193,7 @@ proc ::tk::ConsoleInit {} {
     $w mark set promptEnd insert
     $w mark gravity promptEnd left
 
-    if {$tcl_platform(platform) eq "windows"} {
+    if {[tk windowingsystem] ne "aqua"} {
 	# Subtle work-around to erase the '% ' that tclMain.c prints out
 	after idle [subst -nocommand {
 	    if {[$con get 1.0 output] eq "% "} { $con delete 1.0 output }
@@ -174,7 +215,7 @@ proc ::tk::ConsoleSource {} {
 	    [list [mc "Tcl Scripts"] .tcl] \
 	    [list [mc "All Files"] *]]]
     if {$filename ne ""} {
-    	set cmd [list source $filename]
+	set cmd [list source $filename]
 	if {[catch {consoleinterp eval $cmd} result]} {
 	    ConsoleOutput stderr "$result\n"
 	}
@@ -232,21 +273,22 @@ proc ::tk::ConsoleHistory {cmd} {
     variable HistNum
 
     switch $cmd {
-    	prev {
+	prev {
 	    incr HistNum -1
 	    if {$HistNum == 0} {
 		set cmd {history event [expr {[history nextid] -1}]}
 	    } else {
 		set cmd "history event $HistNum"
 	    }
-    	    if {[catch {consoleinterp eval $cmd} cmd]} {
-    	    	incr HistNum
-    	    	return
-    	    }
+	    if {[catch {consoleinterp eval $cmd} cmd]} {
+		incr HistNum
+		return
+	    }
 	    .console delete promptEnd end
-    	    .console insert promptEnd $cmd {input stdin}
-    	}
-    	next {
+	    .console insert promptEnd $cmd {input stdin}
+	    .console see end
+	}
+	next {
 	    incr HistNum
 	    if {$HistNum == 0} {
 		set cmd {history event [expr {[history nextid] -1}]}
@@ -261,16 +303,17 @@ proc ::tk::ConsoleHistory {cmd} {
 	    }
 	    .console delete promptEnd end
 	    .console insert promptEnd $cmd {input stdin}
-    	}
-    	reset {
-    	    set HistNum 1
-    	}
+	    .console see end
+	}
+	reset {
+	    set HistNum 1
+	}
     }
 }
 
 # ::tk::ConsolePrompt --
 # This procedure draws the prompt.  If tcl_prompt1 or tcl_prompt2
-# exists in the main interpreter it will be called to generate the 
+# exists in the main interpreter it will be called to generate the
 # prompt.  Otherwise, a hard coded default prompt is printed.
 #
 # Arguments:
@@ -281,19 +324,19 @@ proc ::tk::ConsolePrompt {{partial normal}} {
     if {$partial eq "normal"} {
 	set temp [$w index "end - 1 char"]
 	$w mark set output end
-    	if {[consoleinterp eval "info exists tcl_prompt1"]} {
-    	    consoleinterp eval "eval \[set tcl_prompt1\]"
-    	} else {
-    	    puts -nonewline [EvalAttached $::tk::console::defaultPrompt]
-    	}
+	if {[consoleinterp eval "info exists tcl_prompt1"]} {
+	    consoleinterp eval "eval \[set tcl_prompt1\]"
+	} else {
+	    puts -nonewline [EvalAttached $::tk::console::defaultPrompt]
+	}
     } else {
 	set temp [$w index output]
 	$w mark set output end
-    	if {[consoleinterp eval "info exists tcl_prompt2"]} {
-    	    consoleinterp eval "eval \[set tcl_prompt2\]"
-    	} else {
+	if {[consoleinterp eval "info exists tcl_prompt2"]} {
+	    consoleinterp eval "eval \[set tcl_prompt2\]"
+	} else {
 	    puts -nonewline "> "
-    	}
+	}
     }
     flush stdout
     $w mark set output $temp
@@ -304,6 +347,59 @@ proc ::tk::ConsolePrompt {{partial normal}} {
     $w see end
 }
 
+# Copy selected text from the console
+proc ::tk::console::Copy {w} {
+    if {![catch {set data [$w get sel.first sel.last]}]} {
+	clipboard clear -displayof $w
+	clipboard append -displayof $w $data
+    }
+}
+# Copies selected text. If the selection is within the current active edit
+# region then it will be cut, if not it is only copied.
+proc ::tk::console::Cut {w} {
+    if {![catch {set data [$w get sel.first sel.last]}]} {
+	clipboard clear -displayof $w
+	clipboard append -displayof $w $data
+	if {[$w compare sel.first >= output]} {
+	    $w delete sel.first sel.last
+	}
+    }
+}
+# Paste text from the clipboard
+proc ::tk::console::Paste {w} {
+    catch {
+	set clip [::tk::GetSelection $w CLIPBOARD]
+	set list [split $clip \n\r]
+	tk::ConsoleInsert $w [lindex $list 0]
+	foreach x [lrange $list 1 end] {
+	    $w mark set insert {end - 1c}
+	    tk::ConsoleInsert $w "\n"
+	    tk::ConsoleInvoke
+	    tk::ConsoleInsert $w $x
+	}
+    }
+}
+
+# Fit TkConsoleFont to window width
+proc ::tk::console::FitScreenWidth {w} {
+    set width [winfo screenwidth $w]
+    set cwidth [$w cget -width]
+    set s -50
+    set fit 0
+    array set fi [font configure TkConsoleFont]
+    while {$s < 0} {
+	set fi(-size) $s
+	set f [font create {*}[array get fi]]
+	set c [font measure $f "eM"]
+	font delete $f
+	if {$c * $cwidth < 1.667 * $width} {
+	    font configure TkConsoleFont -size $s
+	    break
+	}
+	incr s 2
+    }
+}
+
 # ::tk::ConsoleBind --
 # This procedure first ensures that the default bindings for the Text
 # class have been defined.  Then certain bindings are overridden for
@@ -316,62 +412,89 @@ proc ::tk::ConsoleBind {w} {
     bindtags $w [list $w Console PostConsole [winfo toplevel $w] all]
 
     ## Get all Text bindings into Console
-    foreach ev [bind Text] { bind Console $ev [bind Text $ev] }	
+    foreach ev [bind Text] {
+	bind Console $ev [bind Text $ev]
+    }
     ## We really didn't want the newline insertion...
-    bind Console <Control-Key-o> {}
+    bind Console <Control-o> {}
     ## ...or any Control-v binding (would block <<Paste>>)
-    bind Console <Control-Key-v> {}
+    bind Console <Control-v> {}
 
     # For the moment, transpose isn't enabled until the console
     # gets and overhaul of how it handles input -- hobbs
-    bind Console <Control-Key-t> {}
+    bind Console <Control-t> {}
 
     # Ignore all Alt, Meta, and Control keypresses unless explicitly bound.
     # Otherwise, if a widget binding for one of these is defined, the
+    # <Keypress> class binding will also fire and insert the character
+    # which is wrong.
 
-    bind Console <Alt-KeyPress> {# nothing }
-    bind Console <Meta-KeyPress> {# nothing}
-    bind Console <Control-KeyPress> {# nothing}
+    bind Console <Alt-Key> {# nothing }
+    bind Console <Meta-Key> {# nothing}
+    bind Console <Control-Key> {# nothing}
+    if {[tk windowingsystem] eq "aqua"} {
+	bind Console <Command-Key> {# nothing}
+	bind Console <Mod4-Key> {# nothing}
+    }
 
     foreach {ev key} {
-	<<Console_Prev>>		<Key-Up>
-	<<Console_Next>>		<Key-Down>
-	<<Console_NextImmediate>>	<Control-Key-n>
-	<<Console_PrevImmediate>>	<Control-Key-p>
-	<<Console_PrevSearch>>		<Control-Key-r>
-	<<Console_NextSearch>>		<Control-Key-s>
+	<<Console_NextImmediate>>	<Control-n>
+	<<Console_PrevImmediate>>	<Control-p>
+	<<Console_PrevSearch>>		<Control-r>
+	<<Console_NextSearch>>		<Control-s>
 
-	<<Console_Expand>>		<Key-Tab>
-	<<Console_Expand>>		<Key-Escape>
-	<<Console_ExpandFile>>		<Control-Shift-Key-F>
-	<<Console_ExpandProc>>		<Control-Shift-Key-P>
-	<<Console_ExpandVar>>		<Control-Shift-Key-V>
-	<<Console_Tab>>			<Control-Key-i>
-	<<Console_Tab>>			<Meta-Key-i>
-	<<Console_Eval>>		<Key-Return>
-	<<Console_Eval>>		<Key-KP_Enter>
+	<<Console_Expand>>		<Tab>
+	<<Console_Expand>>		<Escape>
+	<<Console_ExpandFile>>		<Control-Shift-F>
+	<<Console_ExpandProc>>		<Control-Shift-P>
+	<<Console_ExpandVar>>		<Control-Shift-V>
+	<<Console_Tab>>			<Control-i>
+	<<Console_Tab>>			<Meta-i>
+	<<Console_Eval>>		<Return>
+	<<Console_Eval>>		<KP_Enter>
 
-	<<Console_Clear>>		<Control-Key-l>
-	<<Console_KillLine>>		<Control-Key-k>
-	<<Console_Transpose>>		<Control-Key-t>
-	<<Console_ClearLine>>		<Control-Key-u>
-	<<Console_SaveCommand>>		<Control-Key-z>
+	<<Console_Clear>>		<Control-l>
+	<<Console_KillLine>>		<Control-k>
+	<<Console_Transpose>>		<Control-t>
+	<<Console_ClearLine>>		<Control-u>
+	<<Console_SaveCommand>>		<Control-z>
+	<<Console_FontSizeIncr>>	<Control-plus>
+	<<Console_FontSizeDecr>>	<Control-minus>
     } {
 	event add $ev $key
 	bind Console $key {}
     }
-
+    if {[tk windowingsystem] eq "aqua"} {
+	foreach {ev key} {
+	    <<Console_FontSizeIncr>>	<Command-plus>
+	    <<Console_FontSizeDecr>>	<Command-minus>
+	} {
+	    event add $ev $key
+	    bind Console $key {}
+	}
+	if {$::tk::console::useFontchooser} {
+	    bind Console <Command-t> [list ::tk::console::FontchooserToggle]
+	}
+    }
     bind Console <<Console_Expand>> {
-	if {[%W compare insert > promptEnd]} {::tk::console::Expand %W}
+	if {[%W compare insert > promptEnd]} {
+	    ::tk::console::Expand %W
+	}
     }
     bind Console <<Console_ExpandFile>> {
-	if {[%W compare insert > promptEnd]} {::tk::console::Expand %W path}
+	if {[%W compare insert > promptEnd]} {
+	    ::tk::console::Expand %W path
+	}
     }
     bind Console <<Console_ExpandProc>> {
-	if {[%W compare insert > promptEnd]} {::tk::console::Expand %W proc}
+	if {[%W compare insert > promptEnd]} {
+	    ::tk::console::Expand %W proc
+	}
     }
     bind Console <<Console_ExpandVar>> {
-	if {[%W compare insert > promptEnd]} {::tk::console::Expand %W var}
+	if {[%W compare insert > promptEnd]} {
+	    ::tk::console::Expand %W var
+	}
     }
     bind Console <<Console_Eval>> {
 	%W mark set insert {end - 1c}
@@ -380,7 +503,8 @@ proc ::tk::ConsoleBind {w} {
 	break
     }
     bind Console <Delete> {
-	if {[%W tag nextrange sel 1.0 end] ne "" && [%W compare sel.first >= promptEnd]} {
+	if {{} ne [%W tag nextrange sel 1.0 end] \
+		&& [%W compare sel.first >= promptEnd]} {
 	    %W delete sel.first sel.last
 	} elseif {[%W compare insert >= promptEnd]} {
 	    %W delete insert
@@ -388,7 +512,8 @@ proc ::tk::ConsoleBind {w} {
 	}
     }
     bind Console <BackSpace> {
-	if {[%W tag nextrange sel 1.0 end] ne "" && [%W compare sel.first >= promptEnd]} {
+	if {{} ne [%W tag nextrange sel 1.0 end] \
+		&& [%W compare sel.first >= promptEnd]} {
 	    %W delete sel.first sel.last
 	} elseif {[%W compare insert != 1.0] && \
 		[%W compare insert > promptEnd]} {
@@ -398,24 +523,26 @@ proc ::tk::ConsoleBind {w} {
     }
     bind Console <Control-h> [bind Console <BackSpace>]
 
-    bind Console <Home> {
+    bind Console <<LineStart>> {
 	if {[%W compare insert < promptEnd]} {
 	    tk::TextSetCursor %W {insert linestart}
 	} else {
 	    tk::TextSetCursor %W promptEnd
 	}
     }
-    bind Console <Control-a> [bind Console <Home>]
-    bind Console <End> {
+    bind Console <<LineEnd>> {
 	tk::TextSetCursor %W {insert lineend}
     }
-    bind Console <Control-e> [bind Console <End>]
     bind Console <Control-d> {
-	if {[%W compare insert < promptEnd]} break
+	if {[%W compare insert < promptEnd]} {
+	    break
+	}
 	%W delete insert
     }
     bind Console <<Console_KillLine>> {
-	if {[%W compare insert < promptEnd]} break
+	if {[%W compare insert < promptEnd]} {
+	    break
+	}
 	if {[%W compare insert == {insert lineend}]} {
 	    %W delete insert
 	} else {
@@ -455,87 +582,82 @@ proc ::tk::ConsoleBind {w} {
 	    %W delete insert {insert wordend}
 	}
     }
-    bind Console <<Console_Prev>> {
+    bind Console <<PrevLine>> {
 	tk::ConsoleHistory prev
     }
-    bind Console <<Console_Next>> {
+    bind Console <<NextLine>> {
 	tk::ConsoleHistory next
     }
     bind Console <Insert> {
 	catch {tk::ConsoleInsert %W [::tk::GetSelection %W PRIMARY]}
     }
-    bind Console <KeyPress> {
+    bind Console <Key> {
 	tk::ConsoleInsert %W %A
     }
     bind Console <F9> {
-	eval destroy [winfo child .]
-	if {$tcl_platform(platform) eq "macintosh"} {
-	    if {[catch {source [file join $tk_library console.tcl]}]} {source -rsrc console}
-	} else {
-	    source [file join $tk_library console.tcl]
+	destroy {*}[winfo children .]
+	source -encoding utf-8 [file join $tk_library console.tcl]
+    }
+    if {[tk windowingsystem] eq "aqua"} {
+	bind Console <Command-q> {
+	    exit
 	}
     }
-    if {$::tcl_platform(platform) eq "macintosh" || [tk windowingsystem] eq "aqua"} {
-	    bind Console <Command-q> {
-		exit
-	    }
-    }
-    bind Console <<Cut>> {
-        # Same as the copy event
- 	if {![catch {set data [%W get sel.first sel.last]}]} {
-	    clipboard clear -displayof %W
-	    clipboard append -displayof %W $data
+    bind Console <<Cut>> { ::tk::console::Cut %W }
+    bind Console <<Copy>> { ::tk::console::Copy %W }
+    bind Console <<Paste>> { ::tk::console::Paste %W }
+
+    bind Console <<Console_FontSizeIncr>> {
+	set size [font configure TkConsoleFont -size]
+	if {$size < 0} {set sign -1} else {set sign 1}
+	set size [expr {(abs($size) + 1) * $sign}]
+	font configure TkConsoleFont -size $size
+	if {$::tk::console::useFontchooser} {
+	    tk fontchooser configure -font TkConsoleFont
 	}
     }
-    bind Console <<Copy>> {
- 	if {![catch {set data [%W get sel.first sel.last]}]} {
-	    clipboard clear -displayof %W
-	    clipboard append -displayof %W $data
+    bind Console <<Console_FontSizeDecr>> {
+	set size [font configure TkConsoleFont -size]
+	if {abs($size) < 2} { return }
+	if {$size < 0} {set sign -1} else {set sign 1}
+	set size [expr {(abs($size) - 1) * $sign}]
+	font configure TkConsoleFont -size $size
+	if {$::tk::console::useFontchooser} {
+	    tk fontchooser configure -font TkConsoleFont
 	}
     }
-    bind Console <<Paste>> {
-	catch {
-	    set clip [::tk::GetSelection %W CLIPBOARD]
-	    set list [split $clip \n\r]
-	    tk::ConsoleInsert %W [lindex $list 0]
-	    foreach x [lrange $list 1 end] {
-		%W mark set insert {end - 1c}
-		tk::ConsoleInsert %W "\n"
-		tk::ConsoleInvoke
-		tk::ConsoleInsert %W $x
-	    }
-	}
+    bind Console <<Console_FitScreenWidth>> {
+	::tk::console::FitScreenWidth %W
     }
 
     ##
     ## Bindings for doing special things based on certain keys
     ##
-    bind PostConsole <Key-parenright> {
+    bind PostConsole <parenright> {
 	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \( \) promptEnd
 	}
     }
-    bind PostConsole <Key-bracketright> {
+    bind PostConsole <bracketright> {
 	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \[ \] promptEnd
 	}
     }
-    bind PostConsole <Key-braceright> {
+    bind PostConsole <braceright> {
 	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchPair %W \{ \} promptEnd
 	}
     }
-    bind PostConsole <Key-quotedbl> {
+    bind PostConsole <quotedbl> {
 	if {"\\" ne [%W get insert-2c]} {
 	    ::tk::console::MatchQuote %W promptEnd
 	}
     }
 
-    bind PostConsole <KeyPress> {
+    bind PostConsole <Key> {
 	if {"%A" ne ""} {
 	    ::tk::console::TagProc %W
 	}
-	break
     }
 }
 
@@ -554,7 +676,7 @@ proc ::tk::ConsoleInsert {w s} {
 	return
     }
     catch {
-	if {[$w compare sel.first <= insert]
+	if {[$w compare sel.first <= insert] \
 		&& [$w compare sel.last >= insert]} {
 	    $w tag remove sel sel.first promptEnd
 	    $w delete sel.first sel.last
@@ -610,6 +732,35 @@ Tcl $::tcl_patchLevel
 Tk $::tk_patchLevel"
 }
 
+# ::tk::console::Fontchooser* --
+# 	Let the user select the console font (TIP 324).
+
+proc ::tk::console::FontchooserToggle {} {
+    if {[tk fontchooser configure -visible]} {
+	tk fontchooser hide
+    } else {
+	tk fontchooser show
+    }
+}
+proc ::tk::console::FontchooserVisibility {index} {
+    if {[tk fontchooser configure -visible]} {
+	.menubar.edit entryconfigure $index -label [::tk::msgcat::mc "Hide Fonts"]
+    } else {
+	.menubar.edit entryconfigure $index -label [::tk::msgcat::mc "Show Fonts"]
+    }
+}
+proc ::tk::console::FontchooserFocus {w isFocusIn} {
+    if {$isFocusIn} {
+	tk fontchooser configure -parent $w -font TkConsoleFont \
+		-command [namespace code [list FontchooserApply]]
+    } else {
+	tk fontchooser configure -parent $w -font {} -command {}
+    }
+}
+proc ::tk::console::FontchooserApply {font args} {
+    catch {font configure TkConsoleFont {*}[font actual $font]}
+}
+
 # ::tk::console::TagProc --
 #
 # Tags a procedure in the console if it's recognized
@@ -620,10 +771,16 @@ Tk $::tk_patchLevel"
 #	w	- console text widget
 
 proc ::tk::console::TagProc w {
-    if {!$::tk::console::magicKeys} { return }
+    if {!$::tk::console::magicKeys} {
+	return
+    }
     set exp "\[^\\\\\]\[\[ \t\n\r\;{}\"\$\]"
     set i [$w search -backwards -regexp $exp insert-1c promptEnd-1c]
-    if {$i eq ""} {set i promptEnd} else {append i +2c}
+    if {$i eq ""} {
+	set i promptEnd
+    } else {
+	append i +2c
+    }
     regsub -all "\[\[\\\\\\?\\*\]" [$w get $i "insert-1c wordend"] {\\\0} c
     if {[llength [EvalAttached [list info commands $c]]]} {
 	$w tag add proc $i "insert-1c wordend"
@@ -653,32 +810,44 @@ proc ::tk::console::TagProc w {
 # 	c2	- second char of pair
 #
 # Calls:	::tk::console::Blink
- 
+
 proc ::tk::console::MatchPair {w c1 c2 {lim 1.0}} {
-    if {!$::tk::console::magicKeys} { return }
-    if {[set ix [$w search -back $c1 insert $lim]] ne ""} {
+    if {!$::tk::console::magicKeys} {
+	return
+    }
+    if {{} ne [set ix [$w search -back $c1 insert $lim]]} {
 	while {
 	    [string match {\\} [$w get $ix-1c]] &&
-	    [set ix [$w search -back $c1 $ix-1c $lim]] ne ""
+	    [set ix [$w search -back $c1 $ix-1c $lim]] ne {}
 	} {}
 	set i1 insert-1c
-	while {$ix ne ""} {
+	while {$ix ne {}} {
 	    set i0 $ix
 	    set j 0
-	    while {[set i0 [$w search $c2 $i0 $i1]] ne ""} {
+	    while {[set i0 [$w search $c2 $i0 $i1]] ne {}} {
 		append i0 +1c
-		if {[string match {\\} [$w get $i0-2c]]} continue
+		if {[string match {\\} [$w get $i0-2c]]} {
+		    continue
+		}
 		incr j
 	    }
-	    if {!$j} break
+	    if {!$j} {
+		break
+	    }
 	    set i1 $ix
-	    while {$j && [set ix [$w search -back $c1 $ix $lim]] ne ""} {
-		if {[string match {\\} [$w get $ix-1c]]} continue
+	    while {$j && [set ix [$w search -back $c1 $ix $lim]] ne {}} {
+		if {[string match {\\} [$w get $ix-1c]]} {
+		    continue
+		}
 		incr j -1
 	    }
 	}
-	if {[string match {} $ix]} { set ix [$w index $lim] }
-    } else { set ix [$w index $lim] }
+	if {[string match {} $ix]} {
+	    set ix [$w index $lim]
+	}
+    } else {
+	set ix [$w index $lim]
+    }
     if {$::tk::console::blinkRange} {
 	Blink $w $ix [$w index insert]
     } else {
@@ -696,14 +865,20 @@ proc ::tk::console::MatchPair {w c1 c2 {lim 1.0}} {
 #	w	- console text widget
 #
 # Calls:	::tk::console::Blink
- 
+
 proc ::tk::console::MatchQuote {w {lim 1.0}} {
-    if {!$::tk::console::magicKeys} { return }
+    if {!$::tk::console::magicKeys} {
+	return
+    }
     set i insert-1c
     set j 0
-    while {[set i [$w search -back \" $i $lim]] ne ""} {
-	if {[string match {\\} [$w get $i-1c]]} continue
-	if {!$j} {set i0 $i}
+    while {[set i [$w search -back \" $i $lim]] ne {}} {
+	if {[string match {\\} [$w get $i-1c]]} {
+	    continue
+	}
+	if {!$j} {
+	    set i0 $i
+	}
 	incr j
     }
     if {$j&1} {
@@ -771,17 +946,31 @@ proc ::tk::console::ConstrainBuffer {w size} {
 proc ::tk::console::Expand {w {type ""}} {
     set exp "\[^\\\\\]\[\[ \t\n\r\\\{\"\\\\\$\]"
     set tmp [$w search -backwards -regexp $exp insert-1c promptEnd-1c]
-    if {$tmp eq ""} {set tmp promptEnd} else {append tmp +2c}
-    if {[$w compare $tmp >= insert]} { return }
+    if {$tmp eq ""} {
+	set tmp promptEnd
+    } else {
+	append tmp +2c
+    }
+    if {[$w compare $tmp >= insert]} {
+	return
+    }
     set str [$w get $tmp insert]
     switch -glob $type {
-	path* { set res [ExpandPathname $str] }
-	proc* { set res [ExpandProcname $str] }
-	var*  { set res [ExpandVariable $str] }
+	path* {
+	    set res [ExpandPathname $str]
+	}
+	proc* {
+	    set res [ExpandProcname $str]
+	}
+	var* {
+	    set res [ExpandVariable $str]
+	}
 	default {
 	    set res {}
 	    foreach t {Pathname Procname Variable} {
-		if {![catch {Expand$t $str} res] && ($res ne "")} { break }
+		if {![catch {Expand$t $str} res] && ($res ne "")} {
+		    break
+		}
 	    }
 	}
     }
@@ -790,10 +979,12 @@ proc ::tk::console::Expand {w {type ""}} {
 	set repl [lindex $res 0]
 	$w delete $tmp insert
 	$w insert $tmp $repl {input stdin}
-	if {($len > 1) && $::tk::console::showMatches && $repl eq $str} {
+	if {($len > 1) && ($::tk::console::showMatches) && ($repl eq $str)} {
 	    puts stdout [lsort [lreplace $res 0 0]]
 	}
-    } else { bell }
+    } else {
+	bell
+    }
     return [incr len -1]
 }
 
@@ -809,22 +1000,23 @@ proc ::tk::console::Expand {w {type ""}} {
 #
 # Returns:	list containing longest unique match followed by all the
 #		possible further matches
- 
+
 proc ::tk::console::ExpandPathname str {
     set pwd [EvalAttached pwd]
-    if {[catch {EvalAttached [list cd [file dirname $str]]} err]} {
-	return -code error $err
+    if {[catch {EvalAttached [list cd [file dirname $str]]} err opt]} {
+	return -options $opt $err
     }
     set dir [file tail $str]
     ## Check to see if it was known to be a directory and keep the trailing
     ## slash if so (file tail cuts it off)
-    if {[string match */ $str]} { append dir / }
+    if {[string match */ $str]} {
+	append dir /
+    }
     if {[catch {lsort [EvalAttached [list glob $dir*]]} m]} {
 	set match {}
     } else {
 	if {[llength $m] > 1} {
-	    global tcl_platform
-	    if {[string match windows $tcl_platform(platform)]} {
+	    if { $::tcl_platform(platform) eq "windows" } {
 		## Windows is screwy because it's case insensitive
 		set tmp [ExpandBestMatch [string tolower $m] \
 			[string tolower $dir]]
@@ -845,7 +1037,9 @@ proc ::tk::console::ExpandPathname str {
 	} else {
 	    ## This may look goofy, but it handles spaces in path names
 	    eval append match $m
-	    if {[file isdir $match]} {append match /}
+	    if {[file isdir $match]} {
+		append match /
+	    }
 	    if {[string match ?*/* $str]} {
 		set match [file dirname $str]/$match
 	    } elseif {[string match /* $str]} {
@@ -905,12 +1099,14 @@ proc ::tk::console::ExpandProcname str {
 #		possible further matches
 
 proc ::tk::console::ExpandVariable str {
-    if {[regexp {([^\(]*)\((.*)} $str junk ary str]} {
+    if {[regexp {([^\(]*)\((.*)} $str -> ary str]} {
 	## Looks like they're trying to expand an array.
 	set match [EvalAttached [list array names $ary $str*]]
 	if {[llength $match] > 1} {
 	    set vars $ary\([ExpandBestMatch $match $str]
-	    foreach var $match {lappend vars $ary\($var\)}
+	    foreach var $match {
+		lappend vars $ary\($var\)
+	    }
 	    return $vars
 	} elseif {[llength $match] == 1} {
 	    set match $ary\($match\)
@@ -943,8 +1139,8 @@ proc ::tk::console::ExpandVariable str {
 proc ::tk::console::ExpandBestMatch {l {e {}}} {
     set ec [lindex $l 0]
     if {[llength $l]>1} {
-	set e  [string length $e]; incr e -1
-	set ei [string length $ec]; incr ei -1
+	set e [expr {[string length $e] - 1}]
+	set ei [expr {[string length $ec] - 1}]
 	foreach l $l {
 	    while {$ei>=$e && [string first $ec $l]} {
 		set ec [string range $ec 0 [incr ei -1]]
@@ -955,4 +1151,4 @@ proc ::tk::console::ExpandBestMatch {l {e {}}} {
 }
 
 # now initialize the console
-::tk::ConsoleInit 
+::tk::ConsoleInit
